package dataservices;

// internal packages
import communicationservices.*;

// external packages
import java.net.*;
import java.util.*;
import snmp.*;

/**
 * <p>Title: SNMPSession</p>
 * <p>Description: This class is responsible for storing information about a
 * device management consoles (DMCs) that sends requests to the SNMPProxy.
 * It is used by ProtocolHandlers (through the SNMPSessionManager) who wish to
 * que data that was intended for use to respond to a DMC SNMP request but was
 * not availible in time to respond.  This qued data is then used to respond to
 * a repeated requests for that data and then dequed.</p>
 * <p>In addition, a Session is responsible for storing a POMESClient so that
 * each DMC may have its commands trancieved by its own connection to the POMES.</p>
 * <p>Sessions are generally timed out and destroyed by the Session manager
 * after a certain length of time.</p>
 *
 * @see ProtocolThread
 * @see SessionManager
 *
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Printronix</p>
 * @author Christopher M. King (kingc@uci.edu)
 * @version 1.0
 */

public class SNMPSession
    implements SNMPProxyConstants
{
  private InetAddress ip;
  private int port;
  private String managedIP = MANAGED_IP;



  private POMESClient pomesClient;

  public Vector requestBuffer;
  public Vector responseBuffer;
  private boolean pomesAvailable;
  private int TTL = SESSION_TIMEOUT; // time to live
  private SNMPProxyConfig config;

  public SNMPSession(SNMPProxyConfig cfg, InetAddress i)
  {
    ip = i;

    config = cfg;
    requestBuffer = new Vector();
    responseBuffer = new Vector();

    pomesClient = new POMESClient(config);
    pomesAvailable = true;
  }

  public InetAddress getIP()
  {
    return ip;
  }

  public int getPort()
  {
    return port;
  }

  public void setManagedIP(String ip)
  {
    managedIP = ip;
  }

  public String getManagedIP()
  {
    return managedIP;
  }

  public void setPOMESClient(POMESClient pc)
  {
    pomesClient = pc;
  }

  public synchronized POMESClient getPOMESClient()
  {
    resetTTL();
    while(!pomesAvailable)
    { try
      { wait();
      }
      catch(InterruptedException e){}
    }
    if(!pomesClient.POMESSessionOpen())
    { 
      pomesClient.openPOMESSession();
    }

    pomesAvailable = false;
    notifyAll();

    return pomesClient;
  }

  public synchronized void releasePOMESClient()
  { 
    pomesAvailable = true;
    notifyAll();
  }

  public void queSNMPResponse(SNMPMessageInfo response)
  { 
    responseBuffer.add(response);
  }

  public void queSNMPRequest(SNMPMessageInfo request)
  {
   requestBuffer.add(request);
  }

  public boolean containsSNMPRequest(Vector oids)
  {
    resetTTL();
    SNMPMessageInfo curResponse;
    for(int i=0; i<requestBuffer.size(); i++)
    { curResponse = (SNMPMessageInfo)requestBuffer.elementAt(i);
      if(curResponse.getOIDs().equals(oids));
        return true;
    }
    return false;
  }

  public SNMPMessageInfo dequeSNMPRequest(Vector oids)
  {
    resetTTL();
    SNMPMessageInfo curResponse;
    for(int i=0; i<requestBuffer.size(); i++)
    { curResponse = (SNMPMessageInfo)requestBuffer.elementAt(i);
      if(curResponse.getOIDs().equals(oids));
      { requestBuffer.removeElementAt(i);
        return curResponse;
      }
    }
    return null;
  }

  // removes found responses
  public SNMPMessageInfo dequeSNMPResponse(Vector oids)
  {
    resetTTL();
    SNMPMessageInfo curResponse;
    for(int i=0; i<responseBuffer.size(); i++)
    { curResponse = (SNMPMessageInfo)responseBuffer.elementAt(i);
      if(curResponse.getOIDs().toString().equals(oids.toString()))
      {
        responseBuffer.removeElementAt(i);
        return curResponse;
      }
    }
    return null;
  }

 /**
  * Used by the Session manager to decrease the TTL (time-to-live) for this
  * session by 1 second (1000 ms). Only decrements TTL if the POMESClient
  * associated with this session is not being used.
  */
  public void tick()
  {
    if(pomesAvailable)   // <-- prevents timeouts from occurring while the
       TTL = TTL - 1000; //     POMESClient associated with this session is
  }                      //     being used/waited for

  public synchronized void resetTTL()
  {
    TTL = SESSION_TIMEOUT;
  }

 /**
  * Used by the SessionManager to determine if the session has expired.
  *
  *@return True if the session TTL value has reached 0
  */
  public boolean hasExpired()
  {
    return TTL <= 0;
  }
 /**
  * Will return a String representation of the Session, used mostly for debug
  * purposes.
  *
  * @return A String representation of the session contents.
  */
  public String toString()
  {
    String temp = "";

    temp += "Session Open for " + ip;
    temp += " on port " + port + "\n";
    temp += "managing IP: " + managedIP;
    return temp;
  }
}